import java.awt.*;

class Pizarra
{
   protected Console c;
   
   public Pizarra()
   {
      c = new Console();
   }
   
   public Pizarra(int fontSize)
   {
      c = new Console(fontSize);
   }
   
   public Pizarra(int rows, int columns)
   {
      c = new Console(rows, columns);
   }
 
   public Pizarra (int rows, int columns, int fontSize)
   {
      c = new Console(rows, columns, fontSize);
   }
   
   public Pizarra (int rows, int columns, int fontSize, String title)
   {
      c = new Console(rows, columns, fontSize, title);
   }

   public Pizarra (int rows, int columns, String title)
   {     
      c = new Console(rows, columns, title);
   }
   
   public Pizarra (int fontSize, String title)
   {
      c = new Console(fontSize, title);
   }
   
   public Pizarra (String title)
   {
      c = new Console(title);
   }
   
   public void clear ()
   {
      c.clear();
   }
   
   public void close ()
   {
      c.close();
   }


   // Deja un rectangulo en blanco.
   
   public void clearRect (int x1, int y1, int x2, int y2)
   {
      c.clearRect(Math.min(x1,x2), Math.min(maxy()-y1,maxy()-y2), Math.abs(x1-x2), Math.abs(y1-y2));
   }
   
   // Dibuja un arco de elipse de centro (x,y) radios r1 y r2
   // desde el angulo startAngle hasta endAngle (en grados sexagesimales)
   
   public void drawArc (int x, int y, int r1, int r2, int startAngle, int endAngle)
   {
      c.drawArc(x-Math.abs(r1),maxy()/2-(y-Math.abs(r2)), Math.abs(r1)*2, Math.abs(r2)*2, startAngle, endAngle-startAngle);
   }
   
   // traza una linea desde x1,y1 a x2, y2
   public void drawLine (int x1, int y1, int x2, int y2)
   {
      c.drawLine(x1,maxy()-y1,x2,maxy()-y2);
   }
   
   // Dibuja una Elipse de centro (x,y) y radios r1 y r2
   public void drawOval (int x, int y, int r1, int r2)
   {
      c.drawOval(x-Math.abs(r1),maxy()/2-(y-Math.abs(r2)), Math.abs(r1)*2, Math.abs(r2)*2);
   }
   
   // Dados arreglos de Puntos, dibujar el poligono.
   
   public void drawPolygon (int[] xPoints, int[] yPoints, int nPoints)
   {
      int[] yAux = new int[nPoints];
      for(int i=0; i<nPoints; i++)
      {
         yAux[i] = maxy()-yPoints[i];
      }
      c.drawPolygon(xPoints, yAux, nPoints); 
   }
   
   // Dibuja un Rectangulo desde x1,y1 hasta x2, y2
   public void drawRect (int x1, int y1, int x2, int y2)
   {
      c.drawRect(Math.min(x1,x2), Math.min(maxy()-y1,maxy()-y2), Math.abs(x1-x2), Math.abs(y1-y2));
   }
   
   // Coloca un String en la posicion x,y
   public void drawString (String str, int x, int y)
   {
      c.drawString(str, x, maxy()-y);
   }
   
   // Dibuja un arco de elipse relleno.
   public void drawFillArc (int x, int y, int r1, int r2, int startAngle, int endAngle)
   {
      c.fillArc(x-Math.abs(r1),maxy()/2-(y-Math.abs(r2)), Math.abs(r1)*2, Math.abs(r2)*2, startAngle, endAngle-startAngle);
   }
   
   // Dibuja una elipse rellena
   public void drawFillOval(int x, int y, int r1, int r2)
   {
      c.fillOval(x-Math.abs(r1),maxy()/2-(y-Math.abs(r2)), Math.abs(r1)*2, Math.abs(r2)*2);
   }
   
   public void drawFillPolygon (int[] xPoints, int[] yPoints, int nPoints)
   {
      int[] yAux = new int[nPoints];
      for(int i=0; i<nPoints; i++)
      {
         yAux[i] = maxy()-yPoints[i];
      }
      c.fillPolygon(xPoints, yAux, nPoints);
   }
   
   public void drawFillRect (int x1, int y1, int x2, int y2)
   {
      c.fillRect(Math.min(x1,x2), Math.min(maxy()-y1,maxy()-y2), Math.abs(x1-x2), Math.abs(y1-y2));
   }
   
   // Esconde el cursor.
   public void hideCursor ()
   {
      c.hideCursor();
   }
   
   public void showCursor()
   {
      c.showCursor();
   }
   
   // el tamao horizontal de la ventana en pixeles
   public int maxx()
   {
      return c.maxx();
   }
   
   // el tamao vertical de la ventana en pixeles
   public int maxy()
   {
      return c.maxy();
   }
   
   // Dibuja un punto en la posicin x,y
   
   public void drawPoint(int x,int y)
   {
      drawLine(x,y,x,y);
   }

   public void drawTriangle(int x1, int y1, int x2, int y2, int x3, int y3)
   {
      int[] x = new int[3];
      int[] y = new int[3];
      x[0] = x1;
      x[1] = x2;
      x[2] = x3;
      y[0] = y1;
      y[1] = y2;
      y[2] = y3;
      drawPolygon(x, y, 3);
   }
   
   public void drawFillTriangle(int x1, int y1, int x2, int y2, int x3, int y3)
   {
      int[] x = new int[3];
      int[] y = new int[3];
      x[0] = x1;
      x[1] = x2;
      x[2] = x3;
      y[0] = y1;
      y[1] = y2;
      y[2] = y3;
      drawFillPolygon(x, y, 3);
   }
   
   public void setColor(String col)
   {
      if( col == null )
         return;
      else if( col.equalsIgnoreCase("red") )
         c.setColor(Color.red);
      else if( col.equalsIgnoreCase("blue") )
         c.setColor(Color.blue);
      else if( col.equalsIgnoreCase("green") )
         c.setColor(Color.green);
      else if( col.equalsIgnoreCase("yellow") )
         c.setColor(Color.yellow);
      else if( col.equalsIgnoreCase("gray") )
         c.setColor(Color.gray);
      else if( col.equalsIgnoreCase("orange") )
         c.setColor(Color.orange);
      else if( col.equalsIgnoreCase("cyan") )
         c.setColor(Color.cyan);
      else if( col.equalsIgnoreCase("white") )
         c.setColor(Color.white);
      else if( col.equalsIgnoreCase("black") )
         c.setColor(Color.black);
      else if( col.equalsIgnoreCase("magenta") )
         c.setColor(Color.black);
   }
   // Foreground
//   public void setForeColor (Color color)
//   {
//      c.setColor(color);
//  }
   
   // Background
 //  public void setBackColor (Color color)
 //  {
 //     c.setBack

   public static void main(String[] args)
   {
      Pizarra dc = new Pizarra("Demo");
      
      dc.drawFillRect(50,25,100,100);
      dc.drawRect(350,25,200,100);
      
      dc.drawOval(300,300,50,50);
      dc.drawFillOval(400,300,50,50);
      
      dc.drawArc(50,300,50,50,45,90);
      dc.drawFillArc(50,400,50,50,45,120);
      
      dc.clearRect(60,25,70,80);
      dc.drawLine(300,400,250,475);
      dc.drawString("Cymbaline",5,475);
   }
}
