// Cargador de programas para cursos introductorios
// a la programacion.


import tools.*;
import java.lang.reflect.*;
import java.io.*;

public class Run extends Tools {
  // El inicio
  public static void main(String[ ] args) {

    int lastarg;
    boolean graph= false;
    boolean log= false;
    for (lastarg=0; lastarg<args.length; lastarg++) {
      if (args[lastarg].equals("-graph") || args[lastarg].equals("-g"))
        graph= true;
      else if (args[lastarg].equals("-log") || args[lastarg].equals("-l"))
        log= true;
      else
        break;
    }

    if (graph)
      setGraphConsole();
    if (log)
      setLogFile("console.log");

    println("Java Runner ("+version+")");
    println();

    // Verificamos de que se haya especificado el nombre de la clase
    String className;
    boolean pressEnterBeforeQuit;
    if (lastarg<args.length) {
      className= args[lastarg];
      lastarg++;
      pressEnterBeforeQuit= false;
    }
    else {
      print("Ingrese el nombre de su programa por favor: ");
      className= readLine();
      pressEnterBeforeQuit= true;
    }

    // Suprimimos cualquier extension .java o .class que venga en
    // el nombre de la clase
    if ( testSuffix(className,".java") )
      className= className.substring(0, className.length()-".java".length());
    else if ( testSuffix(className,".class") )
      className= className.substring(0, className.length()-".class".length());

    String[ ] trueArgs= new String[args.length-lastarg];
    for (int i=lastarg; i<args.length; i++) trueArgs[i-lastarg]= args[i];

    launch(className, trueArgs, pressEnterBeforeQuit);
  }

  public static void start(String className) {
    String[] emptyArgs= { };
    launch(className, emptyArgs, true);
  }

  static void launch(String className, String[] args,
                     boolean pressEnterBeforeQuit) {
    launch(className, args);
    if (pressEnterBeforeQuit) {
      println();
      println("Presione enter para terminar");
      readLine();
      System.exit(0);
    }
  }

  static void launch(String className, String[] args) {
    // Intentamos cargar la clase
    Class classClass= null;
    try {
      classClass= Class.forName(className);
    }
    catch (ClassNotFoundException excp) {
      println("No se encontro la clase "+className);
      println("Se necesita que su programa se encuentre en alguno de los");
      println("directorios de la variable CLASSPATH.");
      showFormat(className);
      return;
    }
    // Ok, la clase fue encontrada

    // Revisamos que la clase tenga un constructor que no reciba argumentos
    // para poder llamar a newInstance()
    try {
      classClass.getDeclaredConstructor(new Class[0]);
    }
    catch (NoSuchMethodException excp) {
      constructorError(className);
      return;
    }

    Tools prog= null;
    try {
      prog= (Tools)classClass.newInstance();
    }
    catch (InstantiationException excp) {
      println("No se pudo ejecutar la aplicacion porque la clase "+className);
      println("fue definida como abstract o interface.");
      showFormat(className);
      return;
    }
    catch (IllegalAccessException excp) {
      constructorError(className);
      return;
    }
    catch (ClassCastException excp) {
      println("No se pudo ejecutar la aplicacion porque la clase "+className);
      println("no fue definida con extends Program");
      showFormat(className);
    }

    // Vemos si la clase tiene un metodo run con argumentos
    Method method= null;
    Object[] paramArray= null;

    try {
      Class[] paramClassArray= { args.getClass() };
      method= classClass.getDeclaredMethod("run", paramClassArray);
      // Construimos el arreglo de argumentos para la aplicacion
      Object[] paramArray2= { args };
      paramArray= paramArray2;
    }
    catch (NoSuchMethodException excp) {
      // Tratamos de ver si existe run()
      try {
        method= classClass.getDeclaredMethod("run", new Class[0]);
        paramArray= new Object[0];
      }
      catch (NoSuchMethodException excp2) {
        println("No se pudo ejecutar la aplicacion porque no se encontro");
        println("un metodo run() o un metodo run(String[] args).");
        println("");
        showFormat(className);
        return;
      }
    }

    // Invocamos la aplicacion
    try {
      method.invoke(prog, paramArray);
      // Exito
    }
    catch (FinishException excp) {
    }
    catch (Exception excp) {
      println("El programa ha terminado con el siguiente error:");
      Throwable targetExcp= excp;
      if (targetExcp instanceof InvocationTargetException)
        targetExcp=
         ((InvocationTargetException)targetExcp).getTargetException();
      if (targetExcp instanceof ProgramException) {
        String msg= excp.getMessage();
        if (msg==null) msg= "";
        println(msg);
        StringWriter writer= new StringWriter();
        Exception trueExcp= ((ProgramException)targetExcp).getException();
        trueExcp.printStackTrace(new PrintWriter(writer));
        println(writer.toString());
      }
      else {
        StringWriter writer= new StringWriter();
        targetExcp.printStackTrace(new PrintWriter(writer));
        println(writer.toString());
      }
    }
  }

  static boolean testSuffix(String s, String suffix) {
    return suffix.length()<=s.length() &&
           suffix.equals(s.substring(s.length()-suffix.length()));
  }

  static void showFormat(String className) {
    println("");
    println("Su programa debe tener la siguiente forma:");
    println("public "+className+" extends Program {");
    println("  void run() { // o tambien: void run(String[] args)");
    println("    ...");
    println("  }");
    println("  // Otros procedimientos");
    println("  ...");
    println("}");
    println("");
    println("No olvide que el archivo se debe llamar "+className+".java");
  }

  static void constructorError(String className) {
    println("No se pudo ejecutar la aplicacion porque la clase "+className);
    println("no posee un constructor sin argumentos.  Esto ha ocurrido porque");
    println("Ud. ha definido un constructor para la clase "+className+" como");
    println("public "+className+"( ... ) { ... }");
    println("Suprima todos los constructores o bien agregue un constructor que");
    println("no reciba argumentos.");
  }
}
