package tools;

import java.io.*;

public class TokenTextReader extends GenericTextReader {

  BufferedReader in;  // El reader estandar usado
  String fn;          // El nombre del archivo para debugging
  int eofCount= 0;    // Se usa para contar cuantas veces se ha leido
                      // despues del eof
  public static final int MAXEOFCOUNT= 100;

  String line= null;  // Un linea del archivo
  int idx;            // Posicion del ultimo token leido
  int len;            // Largo de la linea

  // Constructor
  public TokenTextReader(String fn) {
    try {
      in= new BufferedReader(new FileReader(fn));
      this.fn= fn;
    }
    catch ( FileNotFoundException excp) {
      throw new FileNotFoundExcp("File "+fn+" not found.");
    }
    catch ( IOException excp ) {
      throw new ProgramException (
        "An input exception happened while opening file "+fn, excp);
    }
  }

  // Constructor
  public TokenTextReader(Reader in, int buffSize) {
    this.in= new BufferedReader(in, buffSize);
    this.fn= "standard input";
  }

  public void close() {
    try {
      in.close();
    }
    catch ( IOException excp ) {
      throw new ProgramException (
        "An input exception happened while closing file "+fn, excp);
    }
  }

  public boolean eofReached() { return eofCount!=0; }

  public String readLine() {
    String ret;
    if (line!=null)
      ret= line.substring(idx);
    else {
      skip();
      ret= line;
    }
    line= null;
    return ret;
  }

  public String readString() {
    do {
      if (line==null)
        skip();
      if (line==null)
        return null;
      trimHead();
    }
      while (line==null && eofCount==0);

    int nextIdx= line.indexOf(" ", idx);
    if (nextIdx==-1) nextIdx= line.indexOf("\t", idx);
    if (nextIdx==-1) nextIdx= len;
    if (idx==nextIdx)
      Tools.fatalError("Internal error while reading:\n"+line);
    String token= line.substring(idx, nextIdx);
    idx= nextIdx;
    trimHead();
    return token;
  }

  public int readInt() {
    String str= readString();
    return eofCount!=0 ? 0 : Integer.parseInt(str);
  }

  public double readDouble() {
    String str= readString();
    return eofCount!=0 ? 0. : new Double(str).doubleValue();
  }

  public boolean readBoolean() {
    String s= Tools.lower(Tools.trim(readString()));
    if (Tools.compare(s,"true")==0)
      return true;
    else if (Tools.compare(s,"false")==0)
      return false;
    else
      Tools.fatalError("readBoolean() has not read true or false from file "
                       +fn);
    return false;
  }

  // Metodos privados
  private void trimHead() {
    while (idx<len) {
      char c= line.charAt(idx);
      if (c!=' ' && c!='\t') break;
      idx++;
    }
    if (idx==len) line= null;
  }

  public void skip() {
    try {
      if (eofCount==0) {
        line= in.readLine(); // Puede lanzar IOException
        if (line==null)
          eofCount= 1;
        else {
          idx= 0;
          len= line.length();
        }
      }
      else if (eofCount>=MAXEOFCOUNT)
        Tools.fatalError(
          "You have read 100 times after the end-of-file.  One read should "+
          "be enough.  You have an infinite loop");
      else
        eofCount++;
    }
    catch ( IOException excp ) {
      throw new ProgramException(
        "An input exception happened while reading file "+fn, excp);
    }
  }

  public boolean hasMoreFields() {
    Tools.fatalError(
      "Method hasMoreFields is not supported when reading by words.");
    return false;
  }
}
